--  Telemetry script: virtual-X-postition-switch for telemetry announcment and display
--  ###################################################################################
--
--  version 4  15.04.2022
--  by ChrisOHara
--  OpenTX version 2.3.11
--  for Horus (but only tested in companion, may run on similar radios as well)
--  This Software is free
--
--  This software works as a kind of virtual X-position-Switch used to display and announce telemetry values
--
--  In the script there is a block with a list of prioritized telemetry values
--    see start of "init function" (you can alter the list as you like)
--  The scripts looks which of these values are available for your active model.
--  The script displays:
--     the name of the selected value,
--     the number of the value  (referring to your list in the script),
--     the value of the selected value;
--     the unit of the selected value;
--  Triggering logical switch L64 ON
--     the script announces the value and unit
--  Triggering logical switch L62 ON
--     the script shifts to the next telemetry value
--  Trigering logical switch L63 ON
--     the script shifts to back to the next telemetry value
--  Announcement and shifting also works as background function without display.
--  You can define the maximum number of values that can be displayed by altering "my_posMax" (default 20)
--  Logical switch examples:  L62 EDGE SFv [0.0:0.3]  0=duration  0=delay
--                            L63 EDGE SFv [0.3:<<]   0=duration  0=delay
--                            L64 OR   SGv SGv
--
--  no garantee, usage at own risk
--
--  New in version 4:
--     bug solved; Announcment of large values now works
--
--########################################################################################################

local my_posMax = 20;      -- defines maximum number of values to be displayed and announced

-- ###########################

local switchNumber = 0;

local ls62_ID = 0;
local ls63_ID = 0;
local ls64_ID = 0;
local ls62LastValue = -1024;
local ls63LastValue = -1024;
local ls64LastValue = -1024;
local my_actualValue = 0;

local my_tele = { name={}, sensorIndex={}, fieldID={}, unitIndex={}, unitText={}, ownListNr={} };
local my_teleAmount = 0;     -- Number of Values to display

local my_xy = { xx={}, yy={}};

local options = {
	{ "Restore", BOOL, 0},
	{ "Text", COLOR, BLACK},
	{ "Warning", COLOR, YELLOW} }

-- ############################# init function ##################
local function TelSxx_create(zone, options)
  -- list of telemetry values to look for
  -- you can change priority by order of lines
  -- you can delete lines
  -- you can add lines as many lines as you like
  -- only the first 10 (my_posMax) sensor found in the modell data will be selected
	my_teleAmount = 0;
	my_tele.name [my_teleAmount] = "A2"   ; my_teleAmount=my_teleAmount+1; 
	my_tele.name [my_teleAmount] = "VFAS" ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "RxBt" ; my_teleAmount=my_teleAmount+1;  
	my_tele.name [my_teleAmount] = "RSSI" ; my_teleAmount=my_teleAmount+1;  
	my_tele.name [my_teleAmount] = "Alt"  ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "Dist" ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "GSpd" ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "Dc2D" ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "RPM"  ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "Erpm" ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "EscA" ; my_teleAmount=my_teleAmount+1;
	my_tele.name [my_teleAmount] = "EscC" ; my_teleAmount=my_teleAmount+1;
  -- end of list

-- looking for sensor in model data
	for i=0, (my_teleAmount -1) do
	   my_tele.sensorIndex [i] = -1;
	   for k=0, 39 do				-- maximum of 40 sensors assumed
              local k_sensor = model.getSensor(k);
	      if k_sensor.name == my_tele.name [i] then
		   my_tele.sensorIndex[i] = k;
		   my_tele.ownListNr[i] = i;
	           break; 
                 end
	     end
	  end

-- eliminating sensor out of list if they were not found
	local kk =0;
	if my_tele.sensorIndex[0] >= 0 then kk=1; end
	for i=1, (my_teleAmount-1) do
	    if (my_tele.sensorIndex[i] >= 0) then
		if (kk ~= i) then
		     my_tele.name[kk] = my_tele.name[i];
		     my_tele.sensorIndex[kk]   = my_tele.sensorIndex[i];
		     my_tele.ownListNr[kk] = my_tele.ownListNr[i];
		  end
		kk = kk + 1;
	      end
          end

-- reducing list of sensor to maximum positions
	if kk > my_posMax then
		my_teleAmount = my_posMax;
	     else
		my_teleAmount = kk;
	     end

-- looking for IDs of the telemetry values
--	local unitData = {[0] = "0-", "1V", "2A", "3mA", "4kn", "5m/s", "6ft/s", "7km/h", "8Mph", "9m",
--				   "10ft", "11C°", "12F°", "13%", "14mAh", "15W", "16mW", "17dB", "18rpm", "19g",
--     	      			   "20deg", "21rad", "22mm", "23Floz", "24Mpm", "25 ", "26 ", "27 ", "28 ", "29 ",
--				   "30 ", "31 ", "32 ", "34 ", "35h", "36min", "37s", "38cells", "39DateTime",
--				   "40GPS", "41bit", "42text"}
	local unitData = {[0] = " ", "V", "A", "mA", "kn", "m/s", "ft/s", "km/h", "Mph", "m",
				   "ft", "C°", "F°", "%", "mAh", "W", "mW", "dB", "rpm", "g",
     	      			   "deg", "rad", "mm", "Floz", "Mpm", " ", " ", " ", " ", " ",
				   " ", " ", " ", " ", "h", "min", "s", "cells", " ",
				   "GPS", "bit", "text"}

	for i=0, (my_teleAmount-1) do
		my_tele.fieldID[i] = getFieldInfo(my_tele.name[i]) and getFieldInfo(my_tele.name[i]).id or nil;
		local dummySensor = model.getSensor(my_tele.sensorIndex[i]);
		my_tele.unitIndex[i] = dummySensor.unit;
	   	my_tele.unitText[i] = unitData[dummySensor.unit];
	     end

	ls62_ID = getFieldInfo("ls62") and getFieldInfo("ls62").id or nil ;
	ls63_ID = getFieldInfo("ls63") and getFieldInfo("ls63").id or nil ;
	ls64_ID = getFieldInfo("ls64") and getFieldInfo("ls64").id or nil ;

-- calcutating coordinates for display
	local TelSxxZone = { zone = zone, options = options } ;
	my_xy.xx[0] = TelSxxZone.zone.x + 0;	my_xy.yy[0] = TelSxxZone.zone.y + 0
	my_xy.xx[1] = TelSxxZone.zone.x + 5;	my_xy.yy[1] = TelSxxZone.zone.y + 20	
	  my_xy.xx[2] = TelSxxZone.zone.x + 100;	my_xy.yy[2] = TelSxxZone.zone.y + 20
	my_xy.xx[3] = TelSxxZone.zone.x + 5;	my_xy.yy[3] = TelSxxZone.zone.y + 50
	  my_xy.xx[4] = TelSxxZone.zone.x + 100;	my_xy.yy[4] = TelSxxZone.zone.y + 50
	return TelSxxZone
end

-- #################### This function allow updates when you change widgets settings #####################
local function TelSxx_update(TelSxxZone, options)
	TelSxxZone.options = options
	return;
end

-- ########################### background function #############################
local function TelSxx_bg(TelSxxZone)
	local newSwitch = switchNumber;

	local ls62NewValue = getValue(ls62_ID);
	if ls62NewValue==1024 and ls62LastValue==-1024 then 
	     newSwitch = newSwitch+1;
	   end
	ls62LastValue = ls62NewValue;


	local ls63NewValue = getValue(ls63_ID);
	if ls63NewValue==1024 and ls63LastValue==-1024 then 
	     newSwitch = newSwitch-1;
	   end
	ls63LastValue = ls63NewValue;

	if newSwitch < 0 then newSwitch = my_teleAmount-1; end
	if newSwitch > my_teleAmount-1 then newSwitch = 0; end

	if newSwitch ~= switchNumber then
		switchNumber = newSwitch;
		playNumber(my_tele.ownListNr[switchNumber],0,0);
	     end

	my_actualValue = getValue(my_tele.fieldID[switchNumber]);
	local ls64NewValue = getValue(ls64_ID);
	if ls64NewValue==1024 and ls64LastValue==-1024 then 
	      if my_actualValue < 100 then 
		  playNumber(my_actualValue*10, my_tele.unitIndex[switchNumber], PREC1 );
		else
		  playNumber(my_actualValue, my_tele.unitIndex[switchNumber], 0 );
		end;
	   end
	ls64LastValue = ls64NewValue;
	return;
end


-- ############################### run function #####################
local function TelSxx_refresh(TelSxxZone)

	TelSxx_bg(TelSxxZone);

	lcd.drawText (my_xy.xx[0],my_xy.yy[0],"Virt. Pos. Switch", MIDSIZE);
        lcd.drawText (my_xy.xx[1],my_xy.yy[1],my_tele.name[switchNumber],  DBLSIZE);
      	  lcd.drawText (my_xy.xx[2],my_xy.yy[2],0.1*math.floor(10*my_actualValue), DBLSIZE);
        lcd.drawText (my_xy.xx[3],my_xy.yy[3],my_tele.ownListNr[switchNumber],  MIDSIZE); 
   	  lcd.drawText (my_xy.xx[4],my_xy.yy[4],my_tele.unitText[switchNumber],  MIDSIZE);

	return 0;
end

return { name = "TelSxx", options=options, create=TelSxx_create, update=TelSxx_update, background= TelSxx_bg, refresh=TelSxx_refresh}

